/*
 * Copyright (c) 1989 Regents of the University of California.
 * All rights reserved.  The Berkeley software License Agreement
 * specifies the terms and conditions for redistribution.
 */

/*
 * pop_getpwnam - fake getpwnam routine by Masanao Izumo <mo@goice.co.jp>
 */

/* Format:
 * user:pass
 */
#define PASSWD_REDIRECT "/etc/pop-passwd"
#define POP_USER "pop"
#define NOSHELL "/bin/true";
#undef getpwnam
#undef getspnam

#include <config.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <pwd.h>
#include <sys/types.h>
#include "popper.h"

static uid_t pop_uid = 0;
static gid_t pop_gid = 0;

static struct passwd *pop_getpwnam_redirect(const char *name)
{
    static struct passwd pwd;
    static char line_buf[MAXLINELEN];
    int n;
    char *id, *pass;
    FILE *fp;

    if(pop_uid == 0)
    {
	static struct passwd *p;
	if((p = getpwnam(POP_USER)) == NULL)
	    return NULL;
	pop_uid = p->pw_uid;
	pop_gid = p->pw_gid;
    }

    if((fp = fopen(PASSWD_REDIRECT, "r")) == NULL)
	return NULL;
    id = line_buf;
    while(fgets(line_buf, sizeof(line_buf), fp) != NULL)
    {
	if((n = strlen(line_buf)) == 0)
	    continue;
	line_buf[n - 1] = '\0';
	if((pass = strchr(line_buf, ':')) == NULL)
	    continue;
	*pass++ = '\0';
	if(strcmp(id, name) != 0)
	    continue;
	pwd.pw_name = id;
	pwd.pw_passwd = pass;
	pwd.pw_uid = pop_uid;
	pwd.pw_gid = pop_gid;
	pwd.pw_age = "";
	pwd.pw_comment = "";
	pwd.pw_gecos = "";
	pwd.pw_dir = "/";
	pwd.pw_shell = NOSHELL;
	fclose(fp);
	return &pwd;
    }
    fclose(fp);
    return NULL;
}

struct passwd *pop_getpwnam(const char *name)
{
    struct passwd *pw;
    if((pw = pop_getpwnam_redirect(name)) == NULL)
	return getpwnam(name);
    return pw;
}

#include <shadow.h>
struct spwd *pop_getspnam(const char *user)
{
    struct passwd *pw;
    static struct spwd spwd;
    if((pw = pop_getpwnam_redirect(user)) == NULL)
	return getspnam(user);
    memset(&spwd, 0, sizeof(spwd));
    spwd.sp_namp = pw->pw_name;
    spwd.sp_pwdp = pw->pw_passwd;
    return &spwd;
}
